
(*################*)
 MODULE TstRegExpr;        (* $VER: tstregexpr.mod 0.0 (20.11.4) *)
(*################*)

FROM ChanConsts IMPORT FlagSet, OpenResults;
FROM ConvTypes  IMPORT ConvResults;
FROM IOChan     IMPORT ChanId;
FROM Strings    IMPORT Assign;
FROM TextIO     IMPORT ReadRestLine, SkipLine, WriteChar, WriteLn, WriteString;
FROM WholeIO    IMPORT WriteInt;
FROM WholeStr   IMPORT StrToCard;
IMPORT STextIO, TermFile;

FROM RegularExpressions  IMPORT CreateNFA, DiscardNFA, NFA, RegExprErrs, RunNFA;

CONST PgmName  = "TstRegExpr";
      PgmVersion = "v0.0 (20.11.4)";

VAR  tty       :ChanId;
     oRes      :OpenResults;
     cRes      :ConvResults;

     RegExpr,
     s         :ARRAY[0..255] OF CHAR;
     Handle    :NFA;
     reErr     :RegExprErrs;

TYPE SampleExpressionsRec = RECORD
                               re               :ARRAY[0..255] OF CHAR;
                               DescriptiveMatch :ARRAY[0..79] OF CHAR;
                            END;
CONST NumSampleExpressions = 5;
VAR  SampleExpressions :ARRAY[0..NumSampleExpressions-1] OF SampleExpressionsRec;
     curSampleExpr     :CARDINAL;

(*---------------------------------------------*)
 PROCEDURE Assert(b:BOOLEAN; msg:ARRAY OF CHAR);
(*---------------------------------------------*)

BEGIN

IF NOT b THEN
   STextIO.WriteString(msg); STextIO.WriteLn;
   STextIO.WriteString("HALTing..."); STextIO.WriteLn;
   HALT;
END;

END Assert;

(*-------------------------------------*)
 PROCEDURE StrClr(VAR s:ARRAY OF CHAR);
(*-------------------------------------*)

BEGIN

s[0] := 0C;
END StrClr;

(*-------------------------------*)
 PROCEDURE InitSampleExpressions;
(*-------------------------------*)

BEGIN

Assign("(A-Z|a-z)#(A-Z|a-z|0-9|_)", SampleExpressions[0].re);   (* eg, identifier names *)
Assign("Identifier_Names", SampleExpressions[0].DescriptiveMatch);

Assign("# '('**'*')", SampleExpressions[1].re); (* comment *)
Assign("(* Comment *)", SampleExpressions[1].DescriptiveMatch);

Assign("([!-'-]|(0-~])#([!-'-]|[0-~])(.|%)#([!-'-]|[0-~])", SampleExpressions[2].re); (* one dot file names *)
Assign("OneDot.filename", SampleExpressions[2].DescriptiveMatch);

Assign("((0|1|%)(0-9|%)(0-9)|2[0-4][0-9]|25[0-5]).((0|1|%)(0-9|%)(0-9)|2[0-4][0-9]|25[0-5]).((0|1|%)(0-9|%)(0-9)|2[0-4][0-9]|25[0-5]).((0|1|%)(0-9|%)(0-9)|2[0-4][0-9]|25[0-5])", SampleExpressions[3].re);
Assign("IPAddress", SampleExpressions[3].DescriptiveMatch);

Assign("(((([0-1]|%)[0-9])|2[0-3]):[0-5][0-9]:[0-5][0-9])|((((((0|%)[1-9])|(1[0-2])):[0-5][0-9]:[0-5][0-9]) (AM|am|PM|pm))|(12:00:00 (midnight|noon)))", SampleExpressions[4].re);
Assign("Time", SampleExpressions[4].DescriptiveMatch);

END InitSampleExpressions;

(*-------------------------------------------------------*)
 PROCEDURE CheckErr(msg:ARRAY OF CHAR; reErr:RegExprErrs);
(*-------------------------------------------------------*)

BEGIN

IF reErr # NoErr THEN
   WriteString(tty, msg); WriteString(tty, "  ");
   WriteChar(tty, CHR(ORD('0') + ORD(reErr)));
END;

END CheckErr;

(*-------------------*)
 PROCEDURE ShowAscii;
(*-------------------*)

VAR  ch :CHAR;

BEGIN

WriteString(tty, "ASCII ordering:");
FOR ch := ' ' TO '0' DO
   WriteChar(tty, ch);
END;
WriteString(tty, "-");
FOR ch := '9' TO 'A' DO
   WriteChar(tty, ch);
END;
WriteString(tty, "-");
FOR ch := 'Z' TO 'a' DO
   WriteChar(tty, ch);
END;
WriteString(tty, "-");
FOR ch := 'z' TO CHR(126) DO
   WriteChar(tty, ch);
END;

END ShowAscii;

(*--------------------*)
 PROCEDURE ShowSyntax;
(*--------------------*)

BEGIN

WriteString(tty, "----------------------------------------------------"); WriteLn(tty);
WriteString(tty, "AlternateOp       '|'"); WriteLn(tty);
WriteString(tty, "ConcatOp             -> implied by adjacency"); WriteLn(tty);
WriteString(tty, "KleeneClosureOp   '#'               ie, 0 or more"); WriteLn(tty);
WriteString(tty, "AnyCharOp         '?'"); WriteLn(tty);
WriteString(tty, 'QuoteOp           "'); WriteString(tty, "'"); WriteString(tty, '"'); WriteLn(tty);
WriteString(tty, "EpsiOp            '%'               ie, null"); WriteLn(tty);
WriteString(tty, "RangeOp           '-'"); WriteLn(tty);
WriteString(tty, "grouping          '(' and ')'  OR   '[' and ']'"); WriteLn(tty);
WriteString(tty, "abbrev for '#?'   '*'"); WriteLn(tty);
WriteString(tty, "----------------------------------------------------");

END ShowSyntax;

(*---------------------*)
 PROCEDURE ShowSamples;
(*---------------------*)

VAR i :INTEGER;

BEGIN
WriteString(tty, "Special aliases for SampleExpressions"); WriteLn(tty);

FOR i := 0 TO VAL(INTEGER, NumSampleExpressions)-1 DO
   WriteString(tty, "   S"); WriteInt(tty, i, 1); WriteString(tty, " is ");
   WriteString(tty, SampleExpressions[i].DescriptiveMatch); WriteLn(tty);
END;

WriteString(tty, "----------------------------------------------------");

END ShowSamples;

(*----------------*)
 BEGIN (* program *)
(*----------------*)

InitSampleExpressions;

TermFile.Open(tty, TermFile.read+TermFile.write, oRes);
Assert(oRes = opened, "TermFile.Open FAIL");

WriteLn(tty);
WriteString(tty, PgmName); WriteString(tty, "   "); WriteString(tty, PgmVersion); WriteLn(tty);
WriteLn(tty);
ShowSyntax;
WriteLn(tty);

(*ShowAscii;*)
ShowSamples;
WriteLn(tty);

REPEAT
   WriteLn(tty); WriteLn(tty);

   WriteString(tty, "RegExpr: ");

   ReadRestLine(tty, RegExpr); SkipLine(tty); WriteLn(tty);

   IF (LENGTH(RegExpr) > 0) THEN

      IF (LENGTH(RegExpr) = 2) AND (RegExpr[0] = "S") THEN
         StrToCard(RegExpr[1], curSampleExpr, cRes);
         IF (cRes = strAllRight) AND (curSampleExpr < NumSampleExpressions) THEN
            Assign(SampleExpressions[curSampleExpr].re, RegExpr);
            WriteString(tty, SampleExpressions[curSampleExpr].DescriptiveMatch); WriteLn(tty);
         END;
      END;

      CreateNFA(Handle, RegExpr, reErr);

      REPEAT
         WriteLn(tty);
         WriteString(tty, "-----------------------------------------"); WriteLn(tty);
         WriteString(tty, RegExpr); WriteLn(tty);
         WriteString(tty, "-----------------------------------------"); WriteLn(tty);

         WriteString(tty, "match: ");
         ReadRestLine(tty, s); SkipLine(tty); WriteLn(tty);

         IF (LENGTH(s) > 0) THEN

            IF RunNFA(Handle, s, reErr) THEN
               WriteString(tty, "   -> TRUE "); WriteLn(tty);
            ELSE
               WriteString(tty, "   -> FALSE"); WriteLn(tty);
            END;
         END;

      UNTIL (LENGTH(s) = 0);

      DiscardNFA(Handle, reErr);
   END;

UNTIL (LENGTH(RegExpr) = 0)

END TstRegExpr.
